package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Consumer;
import java.util.function.Function;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.text.event.ClickEvent;
import xyz.lychee.lagfixer.libs.adventure.text.event.HoverEventSource;
import xyz.lychee.lagfixer.libs.adventure.text.format.MutableStyleSetter;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.adventure.text.format.TextColor;
import xyz.lychee.lagfixer.libs.adventure.text.format.TextDecoration;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style.Merge;
import xyz.lychee.lagfixer.libs.adventure.text.format.TextDecoration.State;
import xyz.lychee.lagfixer.libs.adventure.util.Buildable;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;

@NonExtendable
public interface ComponentBuilder<C extends BuildableComponent<C, B>, B extends ComponentBuilder<C, B>> extends AbstractBuilder<C>, Buildable.Builder<C>, ComponentBuilderApplicable, ComponentLike, MutableStyleSetter<B> {
   @Contract("_ -> this")
   @NotNull B append(final @NotNull Component component);

   @Contract("_ -> this")
   default @NotNull B append(final @NotNull ComponentLike component) {
      return this.append(component.asComponent());
   }

   @Contract("_ -> this")
   default @NotNull B append(final @NotNull ComponentBuilder<?, ?> builder) {
      return this.append((Component)builder.build());
   }

   @Contract("_ -> this")
   @NotNull B append(final @NotNull Component... components);

   @Contract("_ -> this")
   @NotNull B append(final @NotNull ComponentLike... components);

   @Contract("_ -> this")
   @NotNull B append(final @NotNull Iterable<? extends ComponentLike> components);

   default @NotNull B appendNewline() {
      return this.append((Component)Component.newline());
   }

   default @NotNull B appendSpace() {
      return this.append((Component)Component.space());
   }

   @Contract("_ -> this")
   default @NotNull B apply(final @NotNull Consumer<? super ComponentBuilder<?, ?>> consumer) {
      consumer.accept(this);
      return this;
   }

   @Contract("_ -> this")
   @NotNull B applyDeep(final @NotNull Consumer<? super ComponentBuilder<?, ?>> action);

   @Contract("_ -> this")
   @NotNull B mapChildren(final @NotNull Function<BuildableComponent<?, ?>, ? extends BuildableComponent<?, ?>> function);

   @Contract("_ -> this")
   @NotNull B mapChildrenDeep(final @NotNull Function<BuildableComponent<?, ?>, ? extends BuildableComponent<?, ?>> function);

   @NotNull List<Component> children();

   @Contract("_ -> this")
   @NotNull B style(final @NotNull Style style);

   @Contract("_ -> this")
   @NotNull B style(final @NotNull Consumer<Style.Builder> consumer);

   @Contract("_ -> this")
   @NotNull B font(final @Nullable Key font);

   @Contract("_ -> this")
   @NotNull B color(final @Nullable TextColor color);

   @Contract("_ -> this")
   @NotNull B colorIfAbsent(final @Nullable TextColor color);

   @Contract("_, _ -> this")
   default @NotNull B decorations(final @NotNull Set<TextDecoration> decorations, final boolean flag) {
      return (ComponentBuilder)super.decorations(decorations, flag);
   }

   @Contract("_ -> this")
   default @NotNull B decorate(final @NotNull TextDecoration decoration) {
      return this.decoration(decoration, State.TRUE);
   }

   @Contract("_ -> this")
   default @NotNull B decorate(final @NotNull TextDecoration... decorations) {
      return (ComponentBuilder)super.decorate(decorations);
   }

   @Contract("_, _ -> this")
   default @NotNull B decoration(final @NotNull TextDecoration decoration, final boolean flag) {
      return this.decoration(decoration, State.byBoolean(flag));
   }

   @Contract("_ -> this")
   default @NotNull B decorations(final @NotNull Map<TextDecoration, TextDecoration.State> decorations) {
      return (ComponentBuilder)super.decorations(decorations);
   }

   @Contract("_, _ -> this")
   @NotNull B decoration(final @NotNull TextDecoration decoration, final TextDecoration.@NotNull State state);

   @Contract("_, _ -> this")
   @NotNull B decorationIfAbsent(final @NotNull TextDecoration decoration, final TextDecoration.@NotNull State state);

   @Contract("_ -> this")
   @NotNull B clickEvent(final @Nullable ClickEvent event);

   @Contract("_ -> this")
   @NotNull B hoverEvent(final @Nullable HoverEventSource<?> source);

   @Contract("_ -> this")
   @NotNull B insertion(final @Nullable String insertion);

   @Contract("_ -> this")
   default @NotNull B mergeStyle(final @NotNull Component that) {
      return this.mergeStyle(that, Merge.all());
   }

   @Contract("_, _ -> this")
   default @NotNull B mergeStyle(final @NotNull Component that, final Style.@NotNull Merge... merges) {
      return this.mergeStyle(that, Merge.merges(merges));
   }

   @Contract("_, _ -> this")
   @NotNull B mergeStyle(final @NotNull Component that, final @NotNull Set<Style.Merge> merges);

   @NotNull B resetStyle();

   @NotNull C build();

   @Contract("_ -> this")
   default @NotNull B applicableApply(final @NotNull ComponentBuilderApplicable applicable) {
      applicable.componentBuilderApply(this);
      return this;
   }

   default void componentBuilderApply(final @NotNull ComponentBuilder<?, ?> component) {
      component.append(this);
   }

   default @NotNull Component asComponent() {
      return this.build();
   }
}
